window.Gleam.initSearch([{"doc":"omnimessage_lustre","title":"omnimessage_lustre","content":"# omnimessage_lustre\n\n**Seamless server communication using Lustre messages**\n\n[![Package Version](https://img.shields.io/hexpm/v/omnimessage_lustre)](https://hex.pm/packages/omnimessage_lustre)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/omnimessage_lustre/)\n\nA main challenge of maintaining a client side application is state\nsynchronization. Usually, a complete separation is ideal, where client state is\nmanaged exclusively at the client, and server state exclusively at the server.\nThere&#39;s nuance to this take, but it&#39;s a good general rule to follow.\n\n[Lustre](https://hexdocs.pm/lustre/) exceles at this with having both the power\nof a full-blown single page application, and the flexibility of using server\ncomponents for a LiveView/HTMX like solution to state fully owned by the server.\n\nThis approach becomes brittle when the state is shared between both the client\nand server. In a chat app, for example, the messages are state owned by the\nserver, but you still need to display a message as `Sending...` before the\nserver knows it exists. This is usually where LiveView or HTMLX will recommend\nyou sprinkle some Javascript. Lustre has an easier time dealing with this, but\nit can become quite cumbersome to manage the different HTTP requests and\nwebsocket connections.\n\nThis is where Omnimessage comes in. When paired with a properly set up server\n(see [omnimessage_server](https://hexdocs.pm/omnimessage_server)), all you need\nto do to communicate is dispatch messages in Lustre:\n\n```gleam\nimport omnimessage_lustre as omniclient\n\npub fn chat_component() {\n  // Instead of lustre.component, use:\n  omniclient.component(\n    init,\n    update,\n    view,\n    dict.new(),\n    // this is for encoding/decoding, supplied by you\n    encoder_decoder,\n    // this transfers the encoded messages\n    transports.websocket(&quot;http://localhost:8000/omni-app-ws&quot;),\n    TransportState,\n  )\n}\n\n// Divide you messages wisely:\npub type Msg {\n  UserSendDraft\n  UserUpdateDraftMessageContent(content: String)\n  // Messages from the client\n  ClientMessage(ClientMessage)\n  // Messages from the server\n  ServerMessage(ServerMessage)\n  // Messages about transport health\n  TransportState(transports.TransportState(json.DecodeError))\n}\n\n\nfn update(model: Model, msg: Msg) -&gt; #(Model, effect.Effect(Msg)) {\n  case msg {\n    // Merge strategy\n    ServerMessage(shared.ServerUpsertMessages(server_messages)) -&gt; {\n      let messages =\n        model.messages\n        // Omnimessage shines when you&#39;re OK with server being source of truth\n        |&gt; dict.merge(server_messages)\n\n      #(Model(..model, messages:), effect.none())\n    }\n    // ...handle the rest of the messages\n  }\n}\n\n// Then in your view, all you need to do is:\n    html.form()\n      |&gt; event.on_submit(UserSendChat)\n// That message will go to both the client, that can use it to disaply the chat\n// in a sending state, and to the server, which can handle the new message\n// and reply with an updated, correct state.\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/omnimessage_lustre&gt;.\n\n### Thanks &amp; Acknowledgements\n\n- [kero/lustre_websocket](https://codeberg.org/kero/lustre_websocket)\n","url":"index.html"},{"doc":"omnimessage_lustre/transports","title":"Transport","content":"pub type Transport(encoding, decode_error) {\n  Transport(\n    listen: fn(TransportHandlers(encoding, decode_error)) -&gt; Nil,\n    send: fn(encoding, TransportHandlers(encoding, decode_error)) -&gt;\n      Nil,\n  )\n}\n\nTransport(\n  listen: fn(TransportHandlers(encoding, decode_error)) -&gt; Nil,\n  send: fn(encoding, TransportHandlers(encoding, decode_error)) -&gt;\n    Nil,\n)\n\n\nSynonyms:\nomnimessage_lustre/transports.Transport\nomnimessage_lustre/transports Transport","url":"omnimessage_lustre/transports.html#Transport"},{"doc":"omnimessage_lustre/transports","title":"TransportError","content":"pub type TransportError(decode_error) {\n  InitError(message: String)\n  DecodeError(decode_error)\n  SendError(message: String)\n}\n This represents an error in the transport itself (e.g, loss of connection),\n sent inside a `TransportError` record.\n\n Note that this isn&#39;t for error handling of your app logic, use omnimessage\n messages for that.\n\nInitError(message: String)\n\n\nDecodeError(decode_error)\n\n\nSendError(message: String)\n\n\nSynonyms:\nomnimessage_lustre/transports.TransportError\nomnimessage_lustre/transports TransportError","url":"omnimessage_lustre/transports.html#TransportError"},{"doc":"omnimessage_lustre/transports","title":"TransportHandlers","content":"pub type TransportHandlers(encoding, decode_error) {\n  TransportHandlers(\n    on_up: fn() -&gt; Nil,\n    on_down: fn(Int, String) -&gt; Nil,\n    on_message: fn(encoding) -&gt; Nil,\n    on_error: fn(TransportError(decode_error)) -&gt; Nil,\n  )\n}\n Represents the handlers a transport uses for communication. Unless you&#39;re\n building a transport, you don&#39;t need to know about this.\n\nTransportHandlers(\n  on_up: fn() -&gt; Nil,\n  on_down: fn(Int, String) -&gt; Nil,\n  on_message: fn(encoding) -&gt; Nil,\n  on_error: fn(TransportError(decode_error)) -&gt; Nil,\n)\n\n\nSynonyms:\nomnimessage_lustre/transports.TransportHandlers\nomnimessage_lustre/transports TransportHandlers","url":"omnimessage_lustre/transports.html#TransportHandlers"},{"doc":"omnimessage_lustre/transports","title":"TransportState","content":"pub type TransportState(decode_error) {\n  TransportUp\n  TransportDown(code: Int, message: String)\n  TransportError(TransportError(decode_error))\n}\n This type represents the state messages sent to your Lustre application via\n the wrapper you gave on application creation.\n\n It allows you to do housekeeping such as init calls, online/offline\n inidcators, and debugging.\n\nTransportUp\n\n\nTransportDown(code: Int, message: String)\n\n\nTransportError(TransportError(decode_error))\n\n\nSynonyms:\nomnimessage_lustre/transports.TransportState\nomnimessage_lustre/transports TransportState","url":"omnimessage_lustre/transports.html#TransportState"},{"doc":"omnimessage_lustre/transports","title":"http","content":"pub fn http(\n  path path: String,\n  method method: Option(Method),\n  headers headers: Dict(String, String),\n) -&gt; Transport(String, a)\n An http transport using text requests\nSynonyms:\nomnimessage_lustre/transports.http\nomnimessage_lustre/transports http","url":"omnimessage_lustre/transports.html#http"},{"doc":"omnimessage_lustre/transports","title":"websocket","content":"pub fn websocket(path: String) -&gt; Transport(String, a)\n A websocket transport using text frames\nSynonyms:\nomnimessage_lustre/transports.websocket\nomnimessage_lustre/transports websocket","url":"omnimessage_lustre/transports.html#websocket"},{"doc":"omnimessage_lustre/transports","title":"omnimessage_lustre/transports","content":"","url":"omnimessage_lustre/transports.html"},{"doc":"omnimessage_lustre","title":"EncoderDecoder","content":"pub type EncoderDecoder(msg, encoding, decode_error) {\n  EncoderDecoder(\n    encode: fn(msg) -&gt; Result(encoding, Nil),\n    decode: fn(encoding) -&gt; Result(msg, decode_error),\n  )\n}\n Holds decode and encode functions for omnimessage messages. Decode errors\n will be called back for you to handle, while Encode errors are interpreted\n as &quot;skip this message&quot; -- no error will be raised for them and they won&#39;t\n be sent over.\n\n Since an `EncoderDecoder` is expected to receive the whole message type of\n an application, but usually will ignore messages that aren&#39;t shared, it&#39;s\n best to define it as a thin wrapper around shared encoders/decoders:\n\n \n // Holds shared message types, encoders and decoders\n import shared\n\n let encoder_decoder =\n   EncoderDecoder(\n     fn(msg) {\n       case msg {\n         // Messages must be encodable\n         ClientMessage(message) -&gt; Ok(shared.encode_client_message(message))\n         // Return Error(Nil) for messages you don&#39;t want to send out\n         _ -&gt; Error(Nil)\n       }\n     },\n     fn(encoded_msg) {\n       // Unsupported messages will cause TransportError(DecodeError(error))\n       shared.decode_server_message(encoded_msg)\n       |&gt; result.map(ServerMessage)\n     },\n   )\n \n\nEncoderDecoder(\n  encode: fn(msg) -&gt; Result(encoding, Nil),\n  decode: fn(encoding) -&gt; Result(msg, decode_error),\n)\n\n\nSynonyms:\nomnimessage_lustre.EncoderDecoder\nomnimessage_lustre EncoderDecoder","url":"omnimessage_lustre.html#EncoderDecoder"},{"doc":"omnimessage_lustre","title":"application","content":"pub fn application(\n  init: fn(a) -&gt; #(b, Effect(c)),\n  update: fn(d, c) -&gt; #(b, Effect(c)),\n  view: fn(b) -&gt; Element(e),\n  encoder_decoder: EncoderDecoder(c, f, g),\n  transport: Transport(f, g),\n  transport_wrapper: fn(TransportState(g)) -&gt; c,\n) -&gt; App(a, b, e)\n Creates an omnimessage_lustre application. The extra parameters are:\n   - `encoder_decoder`    encodes and decodes messages\n   - `transport`          will transfer and recieve encoded messages. see\n                          `omnimessage_lustre/transports` for available ones\n   - `transport_wrapper`  a wrapper for your `Msg` type for transport status\n\nSynonyms:\nomnimessage_lustre.application\nomnimessage_lustre application","url":"omnimessage_lustre.html#application"},{"doc":"omnimessage_lustre","title":"component","content":"pub fn component(\n  init: fn(a) -&gt; #(b, Effect(c)),\n  update: fn(d, c) -&gt; #(b, Effect(c)),\n  view: fn(b) -&gt; Element(e),\n  on_attribute_change: Dict(\n    String,\n    fn(Dynamic) -&gt; Result(e, List(DecodeError)),\n  ),\n  encoder_decoder: EncoderDecoder(c, f, g),\n  transport: Transport(f, g),\n  transport_wrapper: fn(TransportState(g)) -&gt; c,\n) -&gt; App(a, b, e)\n Creates an omnimessage_lustre Lustre component. The extra parameters are:\n   - `encoder_decoder`    encodes and decodes messages\n   - `transport`          will transfer and recieve encoded messages. see\n                          `omnimessage_lustre/transports` for available ones\n   - `transport_wrapper`  a wrapper for your `Msg` type for transport status\n\nSynonyms:\nomnimessage_lustre.component\nomnimessage_lustre component","url":"omnimessage_lustre.html#component"},{"doc":"omnimessage_lustre","title":"omnimessage_lustre","content":"","url":"omnimessage_lustre.html"}]);